#!/usr/bin/python3

import subprocess
from enum import Enum
import ctypes
from ctypes import *
from pydbus import SystemBus
import syslog
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GLib, Gtk, Gio
import dbus
import dbus.service
import dbus.mainloop.glib
from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)

from kysdk_logsec import kylog
from kdk_accessctl import kyaccessctl
from OfficeProcessMonitor import *

BUS_NAME = "com.kylin.processMonitor"
OBJ_NAME = "/com/kylin/processMonitor"
IFACE_NAME = "com.kylin.processMonitor"


class EmergencyResponseType(Enum):
    EMERGENCY_RESPONSE_LOCK_SCREEN = 1
    EMERGENCY_RESPONSE_REBOOT = 2
    EMERGENCY_RESPONSE_REPAIR_AREA = 3
    EMERGENCY_RESPONSE_PULL_UP_AGAIN = 4

class KeyProcessControl(OfficeProcessMonitor):
    """
    <node>
        <interface name='com.kylin.processMonitor'>
             <method name='setKeyProcess' >
                <arg type='s' direction='in' name='path'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='cancelKeyProcess' >
                <arg type='s' direction='in' name='path'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='getKeyProcess' >
                <arg type='as' direction='out' name='list'/>
            </method>
            <method name='emergencyResponse'>
                <arg type='i' direction='in' name='type'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='processExit' >
                <arg type='s' direction='in' name='path'/>
                <arg type='s' direction='in' name='str'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
             <signal name='signal_processExit' >
                <arg type='s' direction='in' name='path'/>
                <arg type='s' direction='in' name='str'/>
            </signal>
        </interface>
    </node>
    """

    def __init__(self):
        try:
            OfficeProcessMonitor.__init__(self)
            self.kyprocesssec_so = CDLL("/usr/lib/kysdk/kysdk-security/libkyprocesssec.so")
        except Exception as e:
            syslog.syslog(syslog.LOG_ERR, f"KeyProcessControl init error: {e}")
            exit(-1)

    @dbus.service.method(dbus_interface=IFACE_NAME, in_signature="s", out_signature="i", sender_keyword='sender')
    def setKeyProcess(self, path, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "setKeyProcess")
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, kyaccessctl.KdkModuleType.KDK_MODULE_PROCESS.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "setKeyProcess")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        rc = self.kyprocesssec_so.kdk_process_set_key_process(c_char_p(bytes(path, encoding = "utf8")))
        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "setKeyProcess")
        return rc

    @dbus.service.method(dbus_interface=IFACE_NAME, in_signature="s", out_signature="i", sender_keyword='sender')
    def cancelKeyProcess(self, path, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "cancelKeyProcess")
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, kyaccessctl.KdkModuleType.KDK_MODULE_PROCESS.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "cancelKeyProcess")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        rc = self.kyprocesssec_so.kdk_process_cancel_key_process(c_char_p(bytes(path, encoding = "utf8")))
        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "cancelKeyProcess")
        return rc
    
    @dbus.service.method(dbus_interface=IFACE_NAME, in_signature="", out_signature="as", sender_keyword='sender')
    def getKeyProcess(self, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "getKeyProcess")
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, kyaccessctl.KdkModuleType.KDK_MODULE_PROCESS.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "getKeyProcess")
            return []

        list = []
        count = c_int(0)
        self.kyprocesssec_so.kdk_process_get_key_process.restype = POINTER(ctypes.c_char_p)
        keylist = self.kyprocesssec_so.kdk_process_get_key_process(byref(count))
        for i in range(0, count.value):
            list.append(str(keylist[i], "UTF-8"))
        
        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "getKeyProcess")
        return list
    
    def set_emergency_response_lock_screen(self):
        try:
            proxy = SystemBus().get('org.freedesktop.login1', '/org/freedesktop/login1')
            proxy.LockSessions()
            rc = 0
        except Exception as e:
            syslog.syslog(syslog.LOG_ERR, f"Error lock screen: {e}")
            rc = -1
        return rc

    def set_emergency_response_reboot(self):
        try:
            command = ["/usr/sbin/reboot"]
            result = subprocess.run(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
            rc = result.returncode
        except Exception as e:
            syslog.syslog(syslog.LOG_ERR, f"Error execute reboot: {e}")
            rc = -1
        return rc

    @dbus.service.method(dbus_interface=IFACE_NAME, in_signature="i", out_signature="i", sender_keyword='sender')
    def emergencyResponse(self, type, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "emergencyResponse")
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, kyaccessctl.KdkModuleType.KDK_MODULE_PROCESS.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "emergencyResponse")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        if type == EmergencyResponseType.EMERGENCY_RESPONSE_LOCK_SCREEN.value:
            rc = self.set_emergency_response_lock_screen()
        elif type == EmergencyResponseType.EMERGENCY_RESPONSE_REBOOT.value:
            rc = self.set_emergency_response_reboot()
        else:
            rc = -1
        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "emergencyResponse")
        return rc
    
    @dbus.service.method(dbus_interface=IFACE_NAME, in_signature="ss", out_signature="i", sender_keyword='sender')
    def processExit(self, path, str, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "processExit")
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, kyaccessctl.KdkModuleType.KDK_MODULE_PROCESS.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "processExit")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        if len(path) == 0 or len(str) == 0:
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "processExit")
            return -1

        self.signal_processExit(path, str)
        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_PROCESS.value, BUS_NAME, OBJ_NAME, IFACE_NAME, "processExit")
        return 0

    @dbus.service.signal(dbus_interface=IFACE_NAME, signature='ss')
    def signal_processExit(self, path, str):
        syslog.syslog(syslog.LOG_DEBUG, f"emit signal_processExit path:{path}, str:{str}")
        pass

if __name__ == '__main__':
    loop = GLib.MainLoop()
    KeyProcessControl()
    loop.run()
